@@ -48,7 +48,7 @@ def md5sum(filename):
 
 def sha1sum(filename):
     """
-    Calculates md5 hash of a file
+    Calculates sha1 hash of a file
     """
     sha1 = hashlib.sha1()
     with open(filename, 'rb') as f:
@@ -231,11 +231,11 @@ class _ArtifactoryAccessor(pathlib._Accessor):
     """
     Implements operations with Artifactory REST API
     """
-    def rest_get(self, url, headers=None, auth=None):
+    def rest_get(self, url, params=None, headers=None, auth=None):
         """
         Perform a GET request to url with optional authentication
         """
-        res = requests.get(url, headers=headers, auth=auth)
+        res = requests.get(url, params=params, headers=headers, auth=auth)
         return res.text, res.status_code
 
     def rest_put(self, url, params=None, headers=None, auth=None):
@@ -519,6 +519,43 @@ class _ArtifactoryAccessor(pathlib._Accessor):
         if code not in [200, 201]:
             raise RuntimeError("%s" % text)
 
+    def move(self, src, dst):
+        """
+        Move artifact from src to dst
+        """
+        url = '/'.join([src.drive,
+                        'api/move',
+                        str(src.relative_to(src.drive)).rstrip('/')])
+
+        params = {'to': str(dst.relative_to(dst.drive)).rstrip('/')}
+
+        text, code = self.rest_post(url,
+                                    params=params,
+                                    auth=src.auth)
+
+        if code not in [200, 201]:
+            raise RuntimeError("%s" % text)
+
+    def get_properties(self, pathobj):
+        """
+        Get artifact properties and return them as a dictionary.
+        """
+        url = '/'.join([pathobj.drive,
+                        'api/storage',
+                        str(pathobj.relative_to(pathobj.drive)).strip('/')])
+
+        params = 'properties'
+
+        text, code = self.rest_get(url,
+                                   params=params,
+                                   auth=pathobj.auth)
+
+        if code == 404 and "Unable to find item" in text:
+            raise OSError(2, "No such file or directory: '%s'" % url)
+        if code != 200:
+            raise RuntimeError(text)
+
+        return json.loads(text)['properties']
 
 _artifactory_accessor = _ArtifactoryAccessor()
 
@@ -665,6 +702,16 @@ class ArtifactoryPath(pathlib.Path, PureArtifactoryPath):
         obj.auth = self.auth
         return obj
 
+    def __iter__(self):
+        """Iterate over the files in this directory.  Does not yield any
+        result for the special paths '.' and '..'.
+        """
+        for name in self._accessor.listdir(self):
+            if name in {'.', '..'}:
+                # Yielding a path object for these makes little sense
+                continue
+            yield self._make_child_relpath(name)
+
     def open(self, mode='r', buffering=-1, encoding=None,
              errors=None, newline=None):
         """
@@ -828,3 +875,45 @@ class ArtifactoryPath(pathlib.Path, PureArtifactoryPath):
         else:
             with self.open() as fobj:
                 dst.deploy(fobj)
+
+    def move(self, dst):
+        """
+        Move artifact from this path to destinaiton.
+        """
+        if self.drive != dst.drive:
+            raise NotImplementedError(
+                "Moving between instances is not implemented yet")
+
+        self._accessor.move(self, dst)
+
+    @property
+    def properties(self):
+        """
+        Fetch artifact properties
+
+        TODO: implement setting properties
+        """
+        return self._accessor.get_properties(self)
+
+
+def walk(pathobj, topdown=True):
+    """
+    os.walk like function to traverse the URI like a file system.
+
+    The only difference is that this function takes and returns Path objects
+    in places where original implementation will return strings
+    """
+    dirs, nondirs = [], []
+    for child in pathobj.iterdir():
+        if child.is_dir():
+            dirs.append(child)
+        else:
+            nondirs.append(child)
+    if topdown:
+        yield pathobj, dirs, nondirs
+    for dir in dirs:
+        for result in walk(dir):
+            yield result
+    if not topdown:
+        yield pathobj, dirs, nondirs
+
